UNIT QueueFrm;

INTERFACE

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

Type
  TMainForm = class (TForm)
    StatusBar: TStatusBar;
    Panel: TPanel;
    ListBox: TListBox;
    RunButton: TButton;
    SendCheckBox: TCheckBox;
    ReceiveCheckBox: TCheckBox;
    SendEdit: TEdit;
    ReceiveEdit: TEdit;
    procedure RunButtonClick (Sender: TObject);
    procedure SendCheckBoxClick (Sender: TObject);
    procedure ReceiveCheckBoxClick (Sender: TObject);

  private
    procedure Put (s: string);
  end;

Var
  MainForm: TMainForm;

Var send: boolean;
    receive: boolean;

IMPLEMENTATION

Uses SemPro;

{$R *.DFM}

Const max  = 1000*1000; // pocet prenasenych prvku
      info = 1000; // po kolika prenesenych prvcich se zobrazuje informace

      send_group = 10*1000*1000; // po kolika prvcich se ceka
      send_delay = 100; // doba cekani v ms

      receive_group = 10*1000*1000; // po kolika prvcich se ceka
      receive_delay = 100; // doba cekani v ms

Var queue: TQueue;

(********************************* SENDER *********************************)

Type
  TSender = class (TThread)
  private
     start_time: longword;
     stop_time: longword;

     cur: integer;

     procedure ShowTime;
     procedure ShowNumber;

  protected
     procedure Execute; override;

  public
     constructor Create;
  end;

Var Sender: TSender;

Constructor TSender.Create;
Begin
     inherited Create (false);
End;

Procedure TSender.Execute;
Var  i: integer;
     e: TElement;
Begin
     start_time := GetTickCount;

     for i := 1 to max do begin
        repeat until send;

        e.num := i;
        queue.Store (e);
        cur := i;

        if i mod info = 0 then Synchronize (ShowNumber);
        if i mod send_group = 0 then Sleep (send_delay);
     end;

     stop_time := GetTickCount;
     Synchronize (ShowTime);
End;

Procedure TSender.ShowNumber;
Begin
     MainForm.SendEdit.Text := IntToStr (cur);
End;

Procedure TSender.ShowTime;
Var  txt: string;
Begin
     txt := 'Sender ' + IntToStr (stop_time-start_time) + ' ms';
     MainForm.Put (txt);
End;

(******************************** RECEIVER ********************************)

Type
  TReceiver = class (TThread)
  private
     start_time: longword;
     stop_time: longword;

     seq: integer; // ocekavane cislo
     cur: integer; // dosle cislo

     procedure Proceed (num: integer);
     procedure ShowMismatch;
     procedure ShowNumber;
     procedure ShowTime;

  protected
     procedure Execute; override;

  public
     constructor Create;
  end;

Var receiver: TReceiver;

Constructor TReceiver.Create;
Begin
     inherited Create (false);
End;

Procedure TReceiver.Execute;
Var  i: integer;
     e: TElement;
     ready: boolean;
Begin
     start_time := GetTickCount;
     seq := 0;

     for i := 1 to max do begin
        repeat
           { nothing }
        until receive;

        repeat
           queue.Recall (e, ready);
        until ready;

        Proceed (e.num);
        if i mod receive_group = 0 then Sleep (receive_delay);
     end;

     stop_time := GetTickCount;
     Synchronize (ShowTime);
End;

Procedure TReceiver.Proceed (num: integer);
Begin
     cur := num;
     inc (seq);
     if cur <> seq then begin
        send := false;
        Synchronize (ShowMismatch);
        seq := cur;
     end;
     if seq mod info = 0 then Synchronize (ShowNumber);
End;

Procedure TReceiver.ShowMismatch;
Var  txt: string;
Begin
     txt := 'Mismatch ' + IntToStr (seq) + ', ' + IntToStr (cur);
     MainForm.Put (txt);
End;

Procedure TReceiver.ShowNumber;
Begin
     MainForm.ReceiveEdit.Text := IntToStr (seq);
End;

Procedure TReceiver.ShowTime;
Var  txt: string;
Begin
     txt := 'Receiver ' + IntToStr (stop_time-start_time) + ' ms';
     MainForm.Put (txt);

     txt := 'Queue ' + IntToStr (queue.BlockCount) + ' blocks';
     MainForm.Put (txt);
End;

(******************************* MAIN FORM ********************************)

Procedure TMainForm.RunButtonClick (Sender: TObject);
Begin
     ListBox.Items.Clear;
     SendEdit.Text := '';
     ReceiveEdit.Text := '';
     StatusBar.SimpleText := 'Run';
     send := SendCheckBox.checked;
     receive := ReceiveCheckBox.checked;

     queue := TQueue.Create;

     sender := TSender.Create;
     receiver := TReceiver.Create;
End;

Procedure TMainForm.Put (s: string);
Begin
     ListBox.ItemIndex := ListBox.Items.Add (s);
End;

Procedure TMainForm.SendCheckBoxClick (Sender: TObject);
Begin
     send := SendCheckBox.Checked;
End;

Procedure TMainForm.ReceiveCheckBoxClick (Sender: TObject);
Begin
     receive := ReceiveCheckBox.checked;
End;

END.
